@@ -5,15 +5,15 @@
 #  Dirk-jan Mollema (@_dirkjan) -parts of the PKINIT getTGT-
 #
 
-
-import hashlib
+from __future__ import annotations
+from unicrypto import hashlib
 import collections
 import datetime
 import secrets
+from typing import List
 
 from minikerberos import logger
 from minikerberos.common.ccache import CCACHE
-from minikerberos.network.aioclientsocket import AIOKerberosClientSocket
 from minikerberos.protocol.asn1_structs import METHOD_DATA, ETYPE_INFO, ETYPE_INFO2, \
 	PADATA_TYPE, PA_PAC_REQUEST, PA_ENC_TS_ENC, EncryptedData, krb5_pvno, KDC_REQ_BODY, \
 	AS_REQ, TGS_REP, KDCOptions, PrincipalName, EncASRepPart, EncTGSRepPart, PrincipalName, Realm, \
@@ -25,7 +25,7 @@ from minikerberos.protocol.encryption import Key, _enctype_table, _HMACMD5, Enct
 from minikerberos.protocol.constants import PaDataType, EncryptionType, NAME_TYPE, MESSAGE_TYPE
 from minikerberos.protocol.structures import AuthenticatorChecksum
 from minikerberos.gssapi.gssapi import GSSAPIFlags
-from minikerberos.network.selector import KerberosClientSocketSelector
+from minikerberos.network.aioclientsocket import AIOKerberosClientSocket
 
 from minikerberos.common.creds import KerberosCredential
 from minikerberos.common.target import KerberosTarget
@@ -38,7 +38,7 @@ class AIOKerberosClient:
 	def __init__(self, ccred:KerberosCredential, target:KerberosTarget):
 		self.usercreds = ccred
 		self.target = target
-		self.ksoc = KerberosClientSocketSelector.select(self.target, True)
+		self.ksoc = AIOKerberosClientSocket(self.target)
 		self.ccache = CCACHE() if self.usercreds.ccache is None else self.usercreds.ccache
 		self.kerberos_session_key = None
 		self.kerberos_TGT = None
@@ -50,7 +50,7 @@ class AIOKerberosClient:
 		self.server_salt = None
 
 	@staticmethod
-	def from_tgt(target, tgt, key):
+	def from_tgt(target, tgt, key) -> AIOKerberosClient:
 		"""
 		Sets up the kerberos object from tgt and the session key.
 		Use this function when pulling the TGT from ccache file.
@@ -63,7 +63,7 @@ class AIOKerberosClient:
 		kc.kerberos_cipher = _enctype_table[kc.kerberos_cipher_type]
 		return kc
 
-	def build_asreq_lts(self, supported_encryption_method, kdcopts = ['forwardable','renewable','proxiable']):
+	def build_asreq_lts(self, supported_encryption_method, kdcopts:List[str] = ['forwardable','renewable','proxiable']) -> AS_REQ:
 		logger.debug('Constructing TGT request with auth data')
 		#now to create an AS_REQ with encrypted timestamp for authentication
 		pa_data_1 = {}
@@ -106,7 +106,6 @@ class AIOKerberosClient:
 	
 	def build_asreq_pkinit(self, supported_encryption_method, kdcopts = ['forwardable','renewable','renewable-ok']):
 		from asn1crypto import keys
-		import hashlib
 
 		if supported_encryption_method.value == 23:
 			raise Exception('RC4 encryption is not supported for certificate auth!')
@@ -662,9 +661,9 @@ class AIOKerberosClient:
 			ac = AuthenticatorChecksum()
 			ac.flags = flags
 
-			ac.channel_binding = cb_data
-			if cb_data is None:
-				ac.channel_binding = b'\x00'*16
+			ac.channel_binding = b'\x00'*16
+			if cb_data is not None:
+				ac.channel_binding = hashlib.md5(cb_data).digest()
 			
 			chksum = {}
 			chksum['cksumtype'] = 0x8003
@@ -698,11 +697,12 @@ class AIOKerberosClient:
 		authenticator_data['cusec'] = now.microsecond
 		authenticator_data['ctime'] = now.replace(microsecond=0)
 		if flags is not None:
+			
 			ac = AuthenticatorChecksum()
 			ac.flags = flags
-			ac.channel_binding = cb_data
-			if cb_data is None:
-				ac.channel_binding = b'\x00'*16
+			ac.channel_binding = b'\x00'*16
+			if cb_data is not None:
+				ac.channel_binding = hashlib.md5(cb_data).digest()
 			
 			chksum = {}
 			chksum['cksumtype'] = 0x8003
