@@ -3,26 +3,31 @@
 # Author:
 #  Tamas Jos (@skelsec)
 #
-
+from __future__ import annotations
 import getpass
-import hashlib
 import collections
 import base64
 import platform
+from typing import List
+
+
+from unicrypto import hashlib
 
 from minikerberos.common.constants import KerberosSecretType
 from minikerberos.protocol.encryption import string_to_key, Enctype
 from minikerberos.protocol.constants import EncryptionType
 from minikerberos.common.ccache import CCACHE
 from minikerberos.common.keytab import Keytab
-from minikerberos.crypto.hashing import md4
 from asn1crypto import cms
 from asn1crypto import algos
 from minikerberos.protocol.dirtydh import DirtyDH
 
 import oscrypto
 if platform.system().lower() == 'emscripten':
-	oscrypto.use_pure()
+	# these imports are pyodide-specific
+	# the special "openssl" module MUST be imported via JS before attempting this!
+	import ssl
+	oscrypto.use_openssl('/lib/python3.9/site-packages/libcrypto.so', '/lib/python3.9/site-packages/libssl.so')
 
 from oscrypto.asymmetric import rsa_pkcs1v15_sign, load_private_key
 from oscrypto.keys import parse_pkcs12, parse_certificate, parse_private
@@ -31,27 +36,27 @@ from oscrypto.keys import parse_pkcs12, parse_certificate, parse_private
 
 class KerberosCredential:
 	def __init__(self):
-		self.username = None
-		self.domain = None
-		self.password = None
-		self.nt_hash = None
-		self.lm_hash = None
-		self.kerberos_key_aes_256 = None
-		self.kerberos_key_aes_128 = None
-		self.kerberos_key_des = None
-		self.kerberos_key_rc4 = None
-		self.kerberos_key_des3 = None
+		self.username:str = None
+		self.domain:str = None
+		self.password:str = None
+		self.nt_hash:str = None
+		self.lm_hash:str = None
+		self.kerberos_key_aes_256:str = None
+		self.kerberos_key_aes_128:str = None
+		self.kerberos_key_des:str = None
+		self.kerberos_key_rc4:str = None
+		self.kerberos_key_des3:str = None
 		self.certificate = None
 		self.private_key = None
 		self.__hcert = None #handle on the windows certificate store
-		self.__use_windows_certstore = False
-		self.commonname = None
-		self.certstore_name = None
+		self.__use_windows_certstore:bool = False
+		self.commonname:str = None
+		self.certstore_name:str = None
 		self.dhparams:DirtyDH = None
-		self.ccache = None
-		self.ccache_spn_strict_check = True
+		self.ccache:CCACHE = None
+		self.ccache_spn_strict_check:bool = True
 
-	def get_preferred_enctype(self, server_enctypes):
+	def get_preferred_enctype(self, server_enctypes:List[EncryptionType]) -> EncryptionType:
 		client_enctypes = self.get_supported_enctypes(as_int=False)
 		common_enctypes = list(set([s_enctype for s_enctype in server_enctypes]) & set(client_enctypes))
 
@@ -60,12 +65,12 @@ class KerberosCredential:
 				return c_enctype
 
 		raise Exception('No common supported enctypes! Server: %s Client: %s' % (
-			', '.join([s_enctype.name for s_enctype in server_enctypes]),
-			', '.join([c_enctype.name for c_enctype in client_enctypes])
+				', '.join([s_enctype.name for s_enctype in server_enctypes]),
+				', '.join([c_enctype.name for c_enctype in client_enctypes])
+			)
 		)
-						)
 
-	def get_key_for_enctype(self, etype, salt = None):
+	def get_key_for_enctype(self, etype:EncryptionType, salt:bytes = None) -> bytes:
 		"""
 		Returns the encryption key bytes for the enctryption type.
 		"""
@@ -91,8 +96,7 @@ class KerberosCredential:
 			if self.nt_hash:
 				return bytes.fromhex(self.nt_hash)
 			elif self.password:
-				self.nt_hash = md4(self.password.encode('utf-16-le')).hexdigest().upper()
-				#self.nt_hash = hashlib.new('md4', self.password.encode('utf-16-le')).hexdigest().upper()
+				self.nt_hash = hashlib.md4(self.password.encode('utf-16-le')).hexdigest().upper()
 				return bytes.fromhex(self.nt_hash)
 			else:
 				raise Exception('There is no key for RC4 encryption')
@@ -119,7 +123,7 @@ class KerberosCredential:
 		else:
 			raise Exception('Unsupported encryption type: %s' % etype.name)
 
-	def get_supported_enctypes(self, as_int = True):
+	def get_supported_enctypes(self, as_int = True) -> List[EncryptionType]:
 		supp_enctypes = collections.OrderedDict()
 		if self.kerberos_key_aes_256:
 			supp_enctypes[EncryptionType.AES256_CTS_HMAC_SHA1_96] = 1
@@ -152,11 +156,11 @@ class KerberosCredential:
 		return [etype for etype in supp_enctypes]
 	
 	@staticmethod
-	def from_krbcred(keytab_file_path: str, principal: str = None, realm: str = None):
+	def from_krbcred(keytab_file_path: str, principal: str = None, realm: str = None) -> KerberosCredential:
 		return KerberosCredential.from_kirbi(keytab_file_path, principal, realm)
 
 	@staticmethod
-	def from_kirbi(keytab_file_path: str, principal: str = None, realm: str = None):
+	def from_kirbi(keytab_file_path: str, principal: str = None, realm: str = None) -> KerberosCredential:
 		cred = KerberosCredential()
 		cred.username = principal
 		cred.domain = realm
@@ -165,7 +169,7 @@ class KerberosCredential:
 		return cred
 
 	@staticmethod
-	def from_keytab(keytab_file_path: str, principal: str, realm: str):
+	def from_keytab(keytab_file_path: str, principal: str, realm: str) -> KerberosCredential:
 		cred = KerberosCredential()
 		cred.username = principal
 		cred.domain = realm
@@ -199,23 +203,35 @@ class KerberosCredential:
 		return cred
 
 	@staticmethod
-	def from_ccache_file(filepath, principal: str = None, realm: str = None):
+	def from_ccache_file(filepath, principal: str = None, realm: str = None) -> KerberosCredential:
 		k = KerberosCredential()
 		k.username = principal
 		k.domain = realm
 		k.ccache = CCACHE.from_file(filepath)
 		return k
 
-	def set_user_and_domain_from_cert(self, username = None, domain = None):
+	def set_user_and_domain_from_cert(self, username:str = None, domain:str = None):
+		"""
+		Tries to guess the correct username and domain from the current certificate,
+		if 'username' and/or 'domain' is set it will set those
+		"""
 		self.username = username
 		if username is None:
-			self.username = self.certificate.subject.native['common_name'][1]
+			self.username = self.certificate.subject.native['common_name'].rsplit('@', 1)[0]
 		self.domain = domain
 		if domain is None:
-			self.domain = '.'.join(self.certificate.subject.native['domain_component'][::-1])
+			dc = None
+			if 'domain_component' in self.certificate.issuer.native:
+				dc = self.certificate.issuer.native['domain_component']
+			if 'domain_component' in self.certificate.subject.native:
+				dc = self.certificate.subject.native['domain_component']
+			if dc is not None:
+				self.domain = '.'.join(dc[::-1])
+			else:
+				raise Exception('Could\'t find proper domain name in the certificate! Please set it manually!')
 
 	@staticmethod
-	def from_pem_data(certdata, keydata, dhparams = None, username = None, domain = None):
+	def from_pem_data(certdata: str|bytes, keydata:str|bytes, dhparams:DirtyDH = None, username:str = None, domain:str = None) -> KerberosCredential:
 		if isinstance(certdata, str):
 			certdata = base64.b64decode(certdata.replace(' ','').replace('\r','').replace('\n','').replace('\t',''))
 		if isinstance(keydata, str):
@@ -223,11 +239,12 @@ class KerberosCredential:
 		k = KerberosCredential()
 		k.certificate = parse_certificate(certdata)
 		k.private_key = parse_private(keydata)
-		k.set_user_and_domain_from_cert(username = username, domain = username)
+		k.set_user_and_domain_from_cert(username = username, domain = domain)
 		k.set_dhparams(dhparams)
+		return k
 
 	@staticmethod
-	def from_pem_file(certpath, keypath, dhparams = None, username = None, domain = None):
+	def from_pem_file(certpath:str, keypath: str, dhparams:DirtyDH = None, username:str = None, domain:str = None) -> KerberosCredential:
 		with open(certpath, 'rb') as f:
 			certdata = f.read()
 
@@ -238,7 +255,7 @@ class KerberosCredential:
 
 
 	@staticmethod
-	def from_windows_certstore(commonname, certstore_name = 'MY', dhparams = None, username = None, domain = None):
+	def from_windows_certstore(commonname:str, certstore_name:str = 'MY', dhparams:DirtyDH = None, username:str = None, domain:str = None) -> KerberosCredential:
 		if platform.system().lower() != 'windows':
 			raise Exception('Only works on windows (obviously)')
 		from minikerberos.common.windows.crypt32 import find_cert_by_cn, CertCloseStore, CertFreeCertificateContext
@@ -251,12 +268,12 @@ class KerberosCredential:
 		CertCloseStore(shandle)
 
 		k.__use_windows_certstore = True
-		k.set_user_and_domain_from_cert(username = username, domain = username)
+		k.set_user_and_domain_from_cert(username = username, domain = domain)
 		k.set_dhparams(dhparams)
 		return k
 
 	@staticmethod
-	def from_pfx_string(data, password, dhparams = None, username = None, domain = None):
+	def from_pfx_string(data: str|bytes, password:str, dhparams:DirtyDH = None, username:str = None, domain:str = None) -> KerberosCredential:
 		k = KerberosCredential()
 		if password is None:
 			password = b''
@@ -271,12 +288,12 @@ class KerberosCredential:
 		k.private_key, k.certificate, extra_certs = parse_pkcs12(data, password = password)
 		#k.private_key = load_private_key(privkeyinfo)
 		
-		k.set_user_and_domain_from_cert(username = username, domain = username)
+		k.set_user_and_domain_from_cert(username = username, domain = domain)
 		k.set_dhparams(dhparams)
 		return k
 
 	@staticmethod
-	def from_pfx_file(filepath, password, dhparams = None, username = None, domain = None):
+	def from_pfx_file(filepath:str, password:str, dhparams:DirtyDH = None, username:str = None, domain:str = None) -> KerberosCredential:
 		"""
 		Username and domain will override the values found in the certificate
 		"""
